package com.yidianling.consultant.ui.view

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.drawable.BitmapDrawable
import androidx.recyclerview.widget.LinearLayoutManager
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.PopupWindow
import androidx.collection.arraySetOf
import com.tencent.mmkv.MMKV
import com.ydl.ydlcommon.adapter.MyBaseAdapter
import com.ydl.ydlcommon.utils.LogUtil
import com.yidianling.common.tools.RxDeviceTool
import com.yidianling.common.tools.RxImageTool
import com.yidianling.consultant.R
import com.yidianling.consultant.adapter.RegionRecyclerViewAdapter
import com.yidianling.consultant.adapter.SubRecyclerViewAdapter
import com.yidianling.consultant.bean.ConsultantRecentCity
import com.yidianling.consultant.constants.ConsultBIConstants.ConsultantLocationAuth.Companion.RECENT_CITY
import com.yidianling.consultant.model.SearchApi
import com.yidianling.consultant.model.bean.RegionItem
import com.yidianling.consultant.model.bean.SubItem
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.consultant_ui_region_popup_window.view.*

/**
 * 地区选择弹窗
 */
class AreaPopupWindow(
    val context: Context,
    regionList: ArrayList<RegionItem>,
    private var selectedRegion: RegionItem,
    private var selectedSub: SubItem
) : PopupWindow(null, ViewGroup.LayoutParams.MATCH_PARENT, RxImageTool.dp2px(369f)) {

    private val subList = ArrayList<SubItem>()
    private val regionAdapter = RegionRecyclerViewAdapter(context, regionList, selectedRegion)
    private var subAdapter: SubRecyclerViewAdapter

    private var recentCityBean: ConsultantRecentCity? = ConsultantRecentCity(arrayListOf())
    var onRegionSelectedListener: OnRegionSelectedListener? = null
    private var subLocationItem: SubItem? = SubItem()
    private var locationHistoryList: ArrayList<RegionItem>? = arrayListOf()

    init {
        val view =
            LayoutInflater.from(context).inflate(R.layout.consultant_ui_region_popup_window, null)
        this.contentView = view
        this.isFocusable = true
        @Suppress("DEPRECATION")
        this.setBackgroundDrawable(BitmapDrawable())
        this.isOutsideTouchable = true
        this.height =
            ((RxDeviceTool.getScreenHeight(context) - RxImageTool.dp2px(90f)) * 0.618).toInt() //设置高度为屏幕的80%
        //根据IP获取当前城市
        getRegionByIp()
        recentCityBean =
            MMKV.defaultMMKV().decodeParcelable(RECENT_CITY, ConsultantRecentCity::class.java)
        if (recentCityBean == null) recentCityBean = ConsultantRecentCity(arrayListOf())
        var locationList: ArrayList<SubItem>? = arrayListOf()


        locationHistoryList?.add(
            RegionItem(
                recentCityBean!!.recentCity,
                value = "历史",
                key = "0",
                code = "0"
            )
        )
        subLocationItem?.let { locationList?.add(it) }
        locationList?.let { RegionItem(it, value = "定位", key = "-1", code = "-1") }
            ?.let { locationHistoryList?.add(0, it) }

        locationHistoryList?.addAll(regionList)
        regionAdapter.regionList = locationHistoryList!!
        if (selectedRegion.sub.isNotEmpty()) {
            subList.addAll(selectedRegion.sub)
        } else {
            subList.addAll(regionList[0].sub)
        }

        subAdapter = SubRecyclerViewAdapter(context, subList, selectedSub)

        view.rvRegion.layoutManager =
            LinearLayoutManager(
                context,
                LinearLayoutManager.VERTICAL,
                false
            )
        regionAdapter.onItemClickListener = MyBaseAdapter.OnItemClickListener { _, _, data ->
            selectedRegion = data
            subList.clear()
            subList.addAll(data.sub)
            subAdapter.notifyDataSetChanged()
            view.rvSub.scrollToPosition(0)
        }
        view.rvRegion.adapter = regionAdapter
        val i = locationHistoryList!!
            .takeWhile { it.key != selectedRegion.key }
            .count()
        view.rvRegion.scrollToPosition(i)


        view.rvSub.layoutManager =
            LinearLayoutManager(
                context,
                LinearLayoutManager.VERTICAL,
                false
            )
        subAdapter.onItemClickListener = MyBaseAdapter.OnItemClickListener { _, _, data ->
            //倒序保留最近五次的搜索历史记录并去重
            saveCitySearchRecord(data)
            selectedSub = data
            onRegionSelectedListener?.onRegionSelected(selectedRegion, selectedSub)
        }
        view.rvSub.adapter = subAdapter
        view.rvSub.scrollToPosition(subList.indexOf(selectedSub) + 1)
    }

    private fun saveCitySearchRecord(data: SubItem) {
        recentCityBean?.let {
            if (it.recentCity.size < 5) {
                it.recentCity.add(0, data)
                it.recentCity = it.recentCity.toMutableSet().toMutableList()
                MMKV.defaultMMKV().encode(RECENT_CITY, recentCityBean)
            } else {
                it.recentCity.removeAt(it.recentCity.size - 1)
                it.recentCity.add(0, data)
                it.recentCity = it.recentCity.toMutableSet().toMutableList()
                MMKV.defaultMMKV().encode(RECENT_CITY, recentCityBean)
            }
        }
    }

    @SuppressLint("CheckResult")
    fun getRegionByIp() {
        SearchApi.getSearchApi().getCityByIp()
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe({
                if (it.code == "200") {
                    subLocationItem?.code = it.data.city_code
                    subLocationItem?.key = it.data.city_code
                    subLocationItem?.value = it.data.city_name
                }
            }, {})
    }

    interface OnRegionSelectedListener {
        fun onRegionSelected(region: RegionItem, sub: SubItem)
    }
}
